"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const stripAnsi = require("strip-ansi");
const cli_framework_1 = require("@ionic/cli-framework");
const cli_utils_1 = require("@ionic/cli-utils");
const utils_1 = require("../utils");
function default_1(env) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const commands = yield getCommandList(env);
        function listCommandLink(cmd) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const fullName = yield generateFullName(cmd);
                return `[${fullName}](${fullName.split(' ').join('/')}/) | ${cmd.groups && cmd.groups.includes(cli_utils_1.CommandGroup.Deprecated) ? '(deprecated) ' : ''}${stripAnsi(cmd.description)}`;
            });
        }
        const commandLinks = yield Promise.all(commands.map((cmd) => tslib_1.__awaiter(this, void 0, void 0, function* () { return listCommandLink(cmd); })));
        commandLinks.sort();
        return `${utils_1.formatPageHeader('Commands', 'cli-command-list')}

This is a comprehensive list of CLI commands. Running \`ionic --help\` will also show a list of commands.

Command | Description
------- | -----------
${commandLinks.join(`
`)}
`;
    });
}
exports.default = default_1;
function formatCommandHeader(cmd, fullName) {
    return `---
layout: fluid/cli_docs_base
category: cli
id: cli-${fullName.split(' ').join('-')}
page_name: ionic ${fullName}
command_name: ionic ${fullName}
title: ionic ${fullName} - Ionic CLI Documentation
header_sub_title: Ionic CLI
---

${utils_1.getWarning()}

# \`$ ionic ${fullName}\`

`;
}
function formatCommandDoc(env, cmd) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const description = stripAnsi(cmd.description).split('\n').join('\n  ');
        const fullName = yield generateFullName(cmd);
        return formatCommandHeader(cmd, fullName) +
            formatName(fullName, description) +
            formatSynopsis(cmd.inputs || [], fullName) +
            formatDescription(env, cmd) +
            formatExamples(cmd.exampleCommands || [], fullName);
    });
}
exports.formatCommandDoc = formatCommandDoc;
function getCommandList(env) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const cmds = yield env.namespace.getCommandMetadataList();
        return cmds.filter(cmd => !cmd.groups || !cmd.groups.includes(cli_utils_1.CommandGroup.Hidden));
    });
}
exports.getCommandList = getCommandList;
function generateFullName(cmd) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const cmdPath = yield cli_framework_1.generateCommandPath(cmd.command);
        const fullName = cmdPath.map(([p]) => p).slice(1).join(' '); // strip off 'ionic' from beginning
        return fullName;
    });
}
exports.generateFullName = generateFullName;
function formatName(fullName, description) {
    return description;
}
function formatSynopsis(inputs, commandName) {
    const headerLine = `## Synopsis`;
    const usageLine = `${commandName} ${(inputs || [])
        .map(input => {
        return '[<' + input.name + '>]';
    })
        .join(' ')}`;
    return `
${headerLine}

\`\`\`bash
$ ionic ${usageLine}
\`\`\`
  `;
}
function formatDescription(env, cmd) {
    let longDescription = cmd.longDescription;
    if (longDescription) {
        longDescription = stripAnsi(utils_1.links2md(utils_1.ansi2md(longDescription.trim())));
    }
    const inputs = cmd.inputs || [];
    let options = cmd.options || [];
    options = options.filter(o => !o.groups || !o.groups.includes(cli_utils_1.OptionGroup.Hidden));
    const headerLine = `## Details`;
    function inputLineFn(input, index) {
        const name = input.name;
        const description = stripAnsi(utils_1.ansi2md(input.description));
        const optionList = `\`${name}\``;
        return `${optionList} | ${description}`;
    }
    function optionLineFn(option) {
        const showInverse = option.type === Boolean && option.default === true && option.name.length > 1;
        const name = showInverse ? `--no-${option.name}` : `-${option.name.length > 1 ? '-' : ''}${option.name}`;
        const aliases = option.aliases;
        const description = stripAnsi(utils_1.ansi2md(option.description));
        const optionList = `\`${name}\`` +
            (aliases && aliases.length > 0 ? ', ' +
                aliases
                    .map(alias => `\`-${alias}\``)
                    .join(', ') : '');
        return `${optionList} | ${description}`;
    }
    return `
${headerLine}
${longDescription ? '\n' + longDescription + '\n' : ''}
${inputs.length > 0 ? `
Input | Description
----- | ----------` : ``}
${inputs.map(inputLineFn).join(`
`)}

${options.length > 0 ? `
Option | Description
------ | ----------` : ``}
${options.map(optionLineFn).join(`
`)}
`;
}
function formatExamples(exampleCommands, commandName) {
    if (exampleCommands.length === 0) {
        return '';
    }
    const headerLine = `## Examples`;
    const exampleLines = exampleCommands.map(cmd => `$ ionic ${commandName} ${cmd}`);
    return `
${headerLine}

\`\`\`bash
${exampleLines.join('\n')}
\`\`\`
`;
}
